#include "mnthread.h"
#include <stdio.h>

basic_thread::basic_thread(char const* strName, unsigned int uiPriority,
     unsigned short  usStackDepth) {
  m_strName = strName;
  m_uiPriority = uiPriority;
  m_usStackDepth = usStackDepth;
  m_bMutexInit = false;
  m_bRunning = false;

  m_pChild = NULL;
  m_pParent = NULL;
}
basic_thread::~basic_thread() {
  if(!m_bRunning) {delete m_runningMutex; m_bMutexInit =false;}
}
basic_thread*  basic_thread::get_root() {
  basic_thread* root = 0;

  m_contextMutext->lock();
  if(m_pParent != NULL) root = m_pParent->get_root();
  else root = m_pParent;
  m_contextMutext->unlock();

  return root;
}
basic_thread*  basic_thread::get_child() {
  basic_thread* child = 0;

  m_contextMutext->lock();
  child = m_pChild;
  m_contextMutext->unlock();

  return child;
}
bool basic_thread::add_child_thread(basic_thread* thread) {
  bool ret = true;
  if(autolock_t (*m_contextMutext))  {//->lock();
    thread->m_pParent = this;
    if(m_pChild == NULL) m_pChild = thread;
    else ret = m_pChild->add_child_thread(thread);
  }
  return ret;
}
int basic_thread::create(int uiCore) {
  if(!m_bMutexInit) {
    m_runningMutex = new mutext_t();
    m_contextMutext = new mutext_t();

    m_continuemutex = new mutext_t();
    m_continuemutex2 = new mutext_t();

    if(m_runningMutex->create() != ERR_MUTEX_OK)
      return ERR_THREAD_CANTINITMUTEX;
    if(m_contextMutext->create() != ERR_MUTEX_OK)
      return ERR_THREAD_CANTINITMUTEX;

    if(m_continuemutex->create() != ERR_MUTEX_OK)
      return ERR_THREAD_CANTINITMUTEX;
    if(m_continuemutex2->create() != ERR_MUTEX_OK)
      return ERR_THREAD_CANTINITMUTEX;
  }
  m_continuemutex->lock();
	m_runningMutex->lock();
	if (m_bRunning)
	{
		m_runningMutex->unlock();
		m_continuemutex->unlock();
		return ERR_THREAD_ALREADYRUNNING;
	}
	m_runningMutex->unlock();

  if(uiCore == -1) {
    xTaskCreate(&runtaskstub, m_strName,
                 m_usStackDepth,
                this, m_uiPriority, &handle);
    m_iID = get_new_id();
  } else {
    xTaskCreatePinnedToCore(&runtaskstub, m_strName,
                m_usStackDepth,
                this, m_uiPriority, &handle, uiCore);
    m_iID = get_new_id();
  }

	if (handle == 0) {
    m_continuemutex->unlock();
		return ERR_THREAD_CANTSTARTTHREAD;
  }
  m_runningMutex->lock();
  while (!m_bRunning)
	{
		m_runningMutex->unlock();
		sleep(1);
		m_runningMutex->lock();
	}
	m_runningMutex->unlock();
	m_continuemutex->unlock();

	m_continuemutex2->lock();
	m_continuemutex2->unlock();


	return 0;
}
int basic_thread::kill()
{
  m_continuemutex->lock();
  m_runningMutex->lock();

	if (!m_bRunning) {
    m_runningMutex->unlock();
    m_continuemutex->unlock();

		return ERR_THREAD_NOTRUNNING;
	}
  vTaskDelete(handle); handle = 0;
  m_bRunning = false;

  m_runningMutex->unlock();
  m_continuemutex->unlock();

	return 0;
}
bool basic_thread::is_running()
{
	bool r;
  m_runningMutex->lock();
	r = m_bRunning;
  m_runningMutex->unlock();
	return r;
}
uint32_t basic_thread::get_id() {
  uint32_t r;
  m_runningMutex->lock();
	r = m_iID;
  m_runningMutex->unlock();
	return r;
}
const char* basic_thread::get_name() {
  const char*  r;
  m_runningMutex->lock();
	r = m_strName;
  m_runningMutex->unlock();
	return r;
}
unsigned int basic_thread::get_priority() {
  unsigned int r;
  m_runningMutex->lock();
	r = uxTaskPriorityGet(handle);
  m_runningMutex->unlock();
	return r;
}
unsigned short basic_thread::get_stackdepth() {
  unsigned short r;
  m_runningMutex->lock();
	r = m_usStackDepth;
  m_runningMutex->unlock();
	return r;
}
xTaskHandle basic_thread::get_handle() {
  xTaskHandle r;
  m_runningMutex->lock();
	r = handle;
  m_runningMutex->unlock();
	return r;
}
void *basic_thread::get_return_value()
{
	void *val;
  m_runningMutex->lock();
	if (m_bRunning)
		val = NULL;
	else
		val = m_retval;
  m_runningMutex->unlock();
	return val;
}
uint32_t basic_thread::get_time_since_start() {
  uint32_t r;
  m_runningMutex->lock();
	r = (uint32_t)(xTaskGetTickCount()*portTICK_PERIOD_MS);
  m_runningMutex->unlock();
	return r;
}
void  basic_thread::setPriority(unsigned int uiPriority) {
  m_runningMutex->lock();
  vTaskPrioritySet(handle, uiPriority);
  m_runningMutex->unlock();
}
void basic_thread::suspend() {
  m_runningMutex->lock();
  vTaskSuspend( handle );
  m_runningMutex->unlock();
}
void basic_thread::resume() {
  m_runningMutex->lock();
  vTaskResume(handle);
  m_runningMutex->unlock();
}
void basic_thread::runtaskstub(void* parm) {
  basic_thread *esp_thread;
	void *ret;

	esp_thread = (static_cast<basic_thread*>(parm));

  esp_thread->m_continuemutex2->lock();
  esp_thread->m_runningMutex->lock();
	esp_thread->m_bRunning = true;
  esp_thread->m_runningMutex->unlock();

  esp_thread->m_continuemutex->lock();
	esp_thread->m_continuemutex->unlock();

  ret = esp_thread->on_thread();

  esp_thread->m_runningMutex->lock();
	esp_thread->m_bRunning = false;
	esp_thread->m_retval = ret;
  vTaskDelete(esp_thread->handle);
  esp_thread->handle = 0;

  esp_thread->m_runningMutex->unlock();
}

uint32_t __internal_id_base__ = 0;

uint32_t basic_thread::get_new_id() {
  return __internal_id_base__++;
}
